/***************************************************************************/
/*  Copyright 1997 National Instruments Corporation.  All Rights Reserved. */
/***************************************************************************/


/***************************************************************************/
/* There is a possible problem with NI-VISA and events which could         */
/* appear on Win95 and implementations up to 1.2.  If you are              */
/* experiencing problems with viWaitOnEvent timing out , then use          */
/* #define NOVISAEVENT 1, otherwise set this to                            */
/* #define NOVISAEVENT 0   -  or comment out the #define                   */
/***************************************************************************/
/*= ntrt1m ================================================================*/
/* LabWindows Instrument Driver                                            */
/* Modification History:                                                   */
/*                                                                         */
/*         March 1996:   Two occurances of the format spedifier            */
/*                       "%s>%d[b2]/%f %s[xt44]" were replaced by          */
/*                       "%s>%d[b2]/%f %s[xt44t10]" in the readMeas        */
/*						 function. The change was made because of a        */
/*					     linefeed termination of the data returned for     */
/*						 for a single component.						   */ 
/*						 Modified by: K.S., National Instruments           */
/*=========================================================================*/


#define NOVISAEVENT 0                                                     

#include <utility.h>
#include <visa.h>
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <formatio.h>
#include "ntrt1m.h"

/*= DEFINES =================================================================*/
#define ntrt1m_REVISION     "Rev 1.0, 12/97, CVI 4.0.1" /*  Instrument driver revision */
#define BUFFER_SIZE         512L         /* File I/O buffer size */
#define LARGE_BUFFER_SIZE   2048L       /* Large File I/O buffer size */
#define Pi                  3.141592654
    
/*= NEUTRIK RT-1M Audio Test System =========================================*/
/* LabWindows/CVI 4.0.1 Instrument Driver                                    */
/* Original Release: October, 1997                                           */
/* By: DS, National Instruments                                              */
/*     PH. (800) 433-3488   Fax (512) 794-5678                               */
/*                                                                           */
/* Modification History: None                                                */
/*===========================================================================*/
 
/*****************************************************************************/
/*= INSTRUMENT-DEPENDENT COMMAND ARRAYS =====================================*/
/*****************************************************************************/
static ViString fourUnits[] = {"dBVp", "dBV", "Vp", "V"};
static ViString twoUnits1[] = {"dBVp", "Vp"};
static ViString twoUnits2[] = {"dBV", "V"};
static ViString switchs[]   = {"OFF", "ON"};
static ViString swFilters[] = {"OFF", "CWE", "CCIT"};
static ViString selSynch[]  = {"INT", "INTN", "EXT", "EXTN"};
static ViString selMeas[]   = {"LEV", "DIST", "NOIS", "PHAS", "CROS"};
static ViString angleUnits[]= {"RAD","DEG"};
static ViString crossUnits[]= {"%", "dB"};

  
/*****************************************************************************/
/*= INSTRUMENT-DEPENDENT STATUS/RANGE STRUCTURE  ============================*/
/*****************************************************************************/
/* ntrt1m_stringValPair is used in the ntrt1m_errorMessage function          */
/* ntrt1m_statusDataRanges is used to track session dependent status & ranges*/
/*===========================================================================*/
typedef struct  ntrt1m_stringValPair
{
   ViStatus stringVal;
   ViString stringName;
}  ntrt1m_tStringValPair;
 
    /*=CHANGE:=============================================================*/
    /* Change to reflect the global status variables that your driver needs*/
    /* to keep track of.  For example trigger mode of each session         */
    /*=====================================================================*/
struct ntrt1m_statusDataRanges {
    ViInt16 triggerMode;
    ViInt16 val2;
    ViInt16 val3;
    ViChar instrDriverRevision[256];
};

typedef struct ntrt1m_statusDataRanges *ntrt1m_instrRange;

/*****************************************************************************/
/*= UTILITY ROUTINE DECLARATIONS (Non-Exportable Functions) =================*/
/*****************************************************************************/
ViBoolean ntrt1m_invalidViBooleanRange (ViBoolean val);
ViBoolean ntrt1m_invalidViInt16Range (ViInt16 val, ViInt16 min, ViInt16 max);
ViBoolean ntrt1m_invalidViInt32Range (ViInt32 val, ViInt32 min, ViInt32 max);
ViBoolean ntrt1m_invalidViUInt8Range (ViUInt8 val, ViUInt8 min, ViUInt8 max);
ViBoolean ntrt1m_invalidViUInt16Range (ViUInt16 val, ViUInt16 min, ViUInt16 max);
ViBoolean ntrt1m_invalidViUInt32Range (ViUInt32 val, ViUInt32 min, ViUInt32 max);
ViBoolean ntrt1m_invalidViReal32Range (ViReal32 val, ViReal32 min, ViReal32 max);
ViBoolean ntrt1m_invalidViReal64Range (ViReal64 val, ViReal64 min, ViReal64 max);
ViBoolean ntrt1m_invalidBlocklength (ViInt16 val);
ViStatus ntrt1m_waitOnVisaEvent (ViSession instrSession, ViEventType eventTypeIn, ViUInt32 tmoIn, ViPUInt16 STB);
ViStatus ntrt1m_initCleanUp (ViSession openRMSession, ViPSession openInstrSession, ViStatus currentStatus);
ViStatus ntrt1m_readToFile (ViSession instrSession, ViString filename, ViUInt32 readBytes, ViPUInt32 retCount);
ViStatus ntrt1m_writeFromFile (ViSession instrSession, ViString filename, ViUInt32 writeBytes, ViPUInt32 retCount);
ViStatus ntrt1m_get_error (ViSession instrSession);
ViStatus ntrt1m_poll (ViSession instrSession, ViInt16 *);
ViStatus ntrt1m_defaultInstrSetup (ViSession openInstrSession);

/*****************************************************************************/
/*------ INSERT INSTRUMENT-DEPENDENT UTILITY ROUTINE DECLARATIONS HERE ------*/
/*****************************************************************************/


/*****************************************************************************/
/*====== USER-CALLABLE FUNCTIONS (Exportable Functions) =====================*/
/*****************************************************************************/

/*===========================================================================*/
/* Function: Initialize                                                      */
/* Purpose:  This function opens the instrument, queries the instrument      */
/*           for its ID, and initializes the instrument to a known state.    */
/*===========================================================================*/
ViStatus _VI_FUNC ntrt1m_init (ViRsrc resourceName, ViBoolean IDQuery,
                    ViBoolean resetDevice, ViPSession instrSession)
{
    ViStatus ntrt1m_status = VI_SUCCESS;
    ViSession rmSession = 0;
    ViUInt32 retCnt = 0;
    ViByte rdBuffer[BUFFER_SIZE];    

    /*- Check input parameter ranges ----------------------------------------*/
    if (ntrt1m_invalidViBooleanRange (IDQuery))
        return VI_ERROR_PARAMETER2;
    if (ntrt1m_invalidViBooleanRange (resetDevice))
        return VI_ERROR_PARAMETER3;

    /*- Open instrument session ---------------------------------------------*/
    if ((ntrt1m_status = viOpenDefaultRM (&rmSession)) < 0)
        return ntrt1m_status;

    if ((ntrt1m_status = viOpen (rmSession, resourceName, VI_NULL, VI_NULL, instrSession)) < 0) {
        viClose (rmSession);
        return ntrt1m_status;
    }

    /*- Configure VISA Formatted I/O ----------------------------------------*/
    if ((ntrt1m_status = viSetAttribute (*instrSession, VI_ATTR_TMO_VALUE, 10000)) < 0)
            return ntrt1m_initCleanUp (rmSession, instrSession, ntrt1m_status);
  
    if ((ntrt1m_status = viSetBuf (*instrSession, VI_READ_BUF|VI_WRITE_BUF, 4000)) < 0)
            return ntrt1m_initCleanUp (rmSession, instrSession, ntrt1m_status);
  
    if ((ntrt1m_status = viSetAttribute (*instrSession, VI_ATTR_WR_BUF_OPER_MODE,
                            VI_FLUSH_ON_ACCESS)) < 0)
            return ntrt1m_initCleanUp (rmSession, instrSession, ntrt1m_status);
    
    if ((ntrt1m_status = viSetAttribute (*instrSession, VI_ATTR_RD_BUF_OPER_MODE,
                            VI_FLUSH_ON_ACCESS)) < 0)
            return ntrt1m_initCleanUp (rmSession, instrSession, ntrt1m_status);

    /*- Identification Query ------------------------------------------------*/
    if (IDQuery) {
        if ((ntrt1m_status = viWrite (*instrSession, "*IDN?", 5, &retCnt)) < 0)
            return ntrt1m_initCleanUp (rmSession, instrSession, ntrt1m_status);
        if ((ntrt1m_status = viRead (*instrSession, rdBuffer, BUFFER_SIZE, &retCnt)) < 0)
            return ntrt1m_status;

        Scan (rdBuffer, "NEUTRIK,RT1M");
        if (NumFmtdBytes () != 12) 
            return ntrt1m_initCleanUp (rmSession, instrSession, VI_ERROR_FAIL_ID_QUERY);
    }
/*- Reset instrument ----------------------------------------------------*/
    if (resetDevice) {
        if ((ntrt1m_status = ntrt1m_reset (*instrSession)) < 0)
            return ntrt1m_initCleanUp (rmSession, instrSession, ntrt1m_status);
    }       
    else  /*- Send Default Instrument Setup ---------------------------------*/
        if ((ntrt1m_status = ntrt1m_defaultInstrSetup (*instrSession)) < 0)
            return ntrt1m_initCleanUp (rmSession, instrSession, ntrt1m_status);
    return ntrt1m_status;
}

/*===========================================================================*/
/* Function: Simple Measurement                                              */
/* Purpose:  This function sets the input range and the output level with the*/
/*           preferred unit, selects the active signal, and reads the level, */
/*           the distortion, the noise or the phase for the selected channel.*/
/*===========================================================================*/
ViStatus _VI_FUNC ntrt1m_simpleMeas (ViSession instrSession,
                    ViInt16 selectChannel, ViReal64 inputRange, ViReal64 outputLevel, 
                    ViBoolean preferredUnit, ViInt16 activeSignal, ViInt16 query,
                    ViPInt16 numberofBinsQueried, ViInt16 _VI_FAR binNumber[],
                    ViReal64 _VI_FAR amplitude[], ViChar _VI_FAR unit[])
{
    ViStatus ntrt1m_status = VI_SUCCESS;
    ViChar wrtBuf[BUFFER_SIZE];
    ViInt32 retCnt;
    
    if (ntrt1m_invalidViInt16Range (selectChannel, 1, 2))
        return VI_ERROR_PARAMETER2;
    if (ntrt1m_invalidViBooleanRange (preferredUnit))
        return VI_ERROR_PARAMETER5;
    else {
        switch (preferredUnit) {
            case 0:
                if (ntrt1m_invalidViReal64Range (inputRange, -60.0, 20.0))
                    return VI_ERROR_PARAMETER3; 
                if (ntrt1m_invalidViReal64Range (outputLevel, -60.0, 20.0))
                    return VI_ERROR_PARAMETER4; 
                break;
            case 1:
                if (ntrt1m_invalidViReal64Range (inputRange, 0.001, 10.0))
                    return VI_ERROR_PARAMETER3; 
                if (ntrt1m_invalidViReal64Range (outputLevel, 0.001, 10.0))
                    return VI_ERROR_PARAMETER4; 
                break;
        }
    }
    if (ntrt1m_invalidViInt16Range (activeSignal,1,4))
        return VI_ERROR_PARAMETER6;
    if (ntrt1m_invalidViInt16Range (query,0,4))
        return VI_ERROR_PARAMETER7;

    /* Do minimal configuration before signal transmission */
    if ((ntrt1m_status = ntrt1m_configInputRange (instrSession, selectChannel,
                                                  inputRange, preferredUnit)) < 0)
        return ntrt1m_status;
    if ((ntrt1m_status = ntrt1m_configOutputLevel (instrSession, selectChannel,
                                                  outputLevel, preferredUnit)) < 0)
        return ntrt1m_status;
    /* Note that it assumes 0 sec for pretrigger & multitone length */
    if ((ntrt1m_status = ntrt1m_configMtone (instrSession, activeSignal, 0, 0)) < 0)
        return ntrt1m_status;

    /* Now transmit signal */
    if ((ntrt1m_status = ntrt1m_startActiveMtone (instrSession)) < 0)
        return ntrt1m_status;

    /* Query measurements */
    if ((ntrt1m_status = ntrt1m_readMeas (instrSession, selectChannel, query, numberofBinsQueried, 
                                           binNumber, amplitude, unit)) < 0)
        return ntrt1m_status;

    return ntrt1m_status;
}

/*===========================================================================*/
/* Function: Configure Input Mode                                            */
/* Purpose:  This function selects either front or back panel connections for*/
/*           input, links the selected output to the input of the analyzer   */
/*           internally, selects source for synchronization on the expected  */
/*           multitone signal, and enables one of the software filters.      */
/*===========================================================================*/
ViStatus _VI_FUNC ntrt1m_configInputMode (ViSession instrSession,
                    ViBoolean inputConnection, ViBoolean CH1InternalLink,
                    ViBoolean CH2InternalLink, ViInt16 synchronizationMode,
                    ViInt16 softwareWeightingFilter, ViBoolean deemphasisFilter)
{
    ViStatus ntrt1m_status = VI_SUCCESS;
    ViChar wrtBuf[BUFFER_SIZE];
    ViInt32 retCnt;
    
    if (ntrt1m_invalidViBooleanRange (inputConnection))
        return VI_ERROR_PARAMETER2;
    if (ntrt1m_invalidViBooleanRange (CH1InternalLink))
        return VI_ERROR_PARAMETER3;
    if (ntrt1m_invalidViBooleanRange (CH2InternalLink))
        return VI_ERROR_PARAMETER4;
    if (ntrt1m_invalidViInt16Range (synchronizationMode,0,3))
        return VI_ERROR_PARAMETER5;
    if (ntrt1m_invalidViInt16Range (softwareWeightingFilter,0,2))
        return VI_ERROR_PARAMETER6;
    if (ntrt1m_invalidViBooleanRange (deemphasisFilter))
        return VI_ERROR_PARAMETER7;

    Fmt (wrtBuf, "%s<INP:FRON %s;INP1:LINK %s;INP2:LINK %s;INP:SYNC %s;INP:SWF %s;INP:DEEM %s",
            switchs[inputConnection], switchs[CH1InternalLink], switchs[CH2InternalLink],
            selSynch[synchronizationMode], swFilters[softwareWeightingFilter], switchs[deemphasisFilter]);
    if ((ntrt1m_status = viWrite (instrSession, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)
        return ntrt1m_status;

    return ntrt1m_status;
}

/*===========================================================================*/
/* Function: Configure Input Range                                           */
/* Purpose:  This function sets the input range and preferred unit for the   */
/*           selected channel.                                               */
/*===========================================================================*/
ViStatus _VI_FUNC ntrt1m_configInputRange (ViSession instrSession,
                                           ViInt16 input, ViReal64 inputRange,
                                           ViBoolean inputUnit)
{
    ViStatus ntrt1m_status = VI_SUCCESS;
    ViChar wrtBuf[BUFFER_SIZE];
    ViInt32 retCnt;
    
    if (ntrt1m_invalidViInt16Range (input,0,2))
        return VI_ERROR_PARAMETER2;
    if (ntrt1m_invalidViBooleanRange (inputUnit))
        return VI_ERROR_PARAMETER4;
    else {
        switch (inputUnit) {
            case 0:
                if (ntrt1m_invalidViReal64Range (inputRange, -60.0, 20.0))
                    return VI_ERROR_PARAMETER3; 
                break;
            case 1:
                if (ntrt1m_invalidViReal64Range (inputRange, 0.001, 10.0))
                    return VI_ERROR_PARAMETER3; 
                break;
        }
    }
    switch (input) {
        case 0:
            Fmt (wrtBuf, "%s<INP1:RANG %f %s;INP2:RANG %f %s",
                inputRange, twoUnits1[inputUnit], inputRange, twoUnits1[inputUnit]);
            break;
        case 1:
            Fmt (wrtBuf, "%s<INP1:RANG %f %s", inputRange, twoUnits1[inputUnit]);
            break;
        case 2:
            Fmt (wrtBuf, "%s<INP2:RANG %f %s", inputRange, twoUnits1[inputUnit]);
            break;
    }
    if ((ntrt1m_status = viWrite (instrSession, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)
        return ntrt1m_status;

    return ntrt1m_status;
}

/*===========================================================================*/
/* Function: Configure Output Mode                                           */
/* Purpose:  This function selects either floating or grounded mode for both */
/*           output channels, and mutes the selected output channels.        */
/*===========================================================================*/
ViStatus _VI_FUNC ntrt1m_configOutputMode (ViSession instrSession,
                                           ViBoolean floatGroundMode,
                                           ViBoolean Channel1Mute, 
                                           ViBoolean Channel2Mute)
{
    ViStatus ntrt1m_status = VI_SUCCESS;
    ViChar wrtBuf[BUFFER_SIZE];
    ViInt32 retCnt;
    
    if (ntrt1m_invalidViBooleanRange (floatGroundMode))
        return VI_ERROR_PARAMETER2;
    if (ntrt1m_invalidViBooleanRange (Channel1Mute))
        return VI_ERROR_PARAMETER3;
    if (ntrt1m_invalidViBooleanRange (Channel2Mute))
        return VI_ERROR_PARAMETER4;
    Fmt (wrtBuf, "%s<OUTP:FLOAT %s;OUTP1:MUT %s;OUTP2:MUT %s",
            switchs[floatGroundMode], switchs[Channel1Mute], switchs[Channel2Mute]);
    if ((ntrt1m_status = viWrite (instrSession, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)
        return ntrt1m_status;

    return ntrt1m_status;
}

/*===========================================================================*/
/* Function: Configure Output Level                                          */
/* Purpose:  This function sets the output level and preferred unit for the  */
/*           selected channel.                                               */
/*===========================================================================*/
ViStatus _VI_FUNC ntrt1m_configOutputLevel (ViSession instrSession,
                                            ViInt16 Output, ViReal64 outputLevel, 
                                            ViBoolean outputUnit)
{
    ViStatus ntrt1m_status = VI_SUCCESS;
    ViChar wrtBuf[BUFFER_SIZE];
    ViInt32 retCnt;
    
    if (ntrt1m_invalidViInt16Range (Output,0,2))
        return VI_ERROR_PARAMETER2;
    if (ntrt1m_invalidViBooleanRange (outputUnit))
        return VI_ERROR_PARAMETER4;
    else {
        switch (outputUnit) {
            case 0:
                if (ntrt1m_invalidViReal64Range (outputLevel, -60.0, 20.0))
                    return VI_ERROR_PARAMETER3; 
                break;
            case 1:
                if (ntrt1m_invalidViReal64Range (outputLevel, 0.001, 10.0))
                    return VI_ERROR_PARAMETER3; 
                break;
        }
    }
    switch (Output) {
        case 0:
            Fmt (wrtBuf, "%s<OUTP1:LEV %f %s;OUTP2:LEV %f %s",
                outputLevel, twoUnits1[outputUnit], outputLevel, twoUnits1[outputUnit]);
            break;
        case 1:
            Fmt (wrtBuf, "%s<OUTP1:LEV %f %s", outputLevel, twoUnits1[outputUnit]);
            break;
        case 2:
            Fmt (wrtBuf, "%s<OUTP2:LEV %f %s", outputLevel, twoUnits1[outputUnit]);
            break;
    }
    if ((ntrt1m_status = viWrite (instrSession, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)
        return ntrt1m_status;

    return ntrt1m_status;
}

/*===========================================================================*/
/* Function: Configure Measurement Unit                                      */
/* Purpose:  This function selects preferred unit for level, distortion, and */
/*           noise value of the selected channel.                            */                   
/*===========================================================================*/
ViStatus _VI_FUNC ntrt1m_configMeasUnit (ViSession instrSession,
                    ViInt16 configure, ViInt16 levelUnit,
                    ViBoolean distortionUnit, ViBoolean noiseUnit, 
                    ViBoolean phaseUnit, ViReal64 phaseScale, ViBoolean crosstalk)
{
    ViStatus ntrt1m_status = VI_SUCCESS;
    ViChar wrtBuf[BUFFER_SIZE];
    ViInt32 retCnt;
    
    if (ntrt1m_invalidViInt16Range (configure,0,2))
        return VI_ERROR_PARAMETER2;
    if (ntrt1m_invalidViInt16Range (levelUnit,0,3))
        return VI_ERROR_PARAMETER3;
    if (ntrt1m_invalidViBooleanRange (distortionUnit))
        return VI_ERROR_PARAMETER4;
    if (ntrt1m_invalidViBooleanRange (noiseUnit))
        return VI_ERROR_PARAMETER5;
    if (ntrt1m_invalidViBooleanRange (phaseUnit))
        return VI_ERROR_PARAMETER6;
    switch (phaseUnit) {
        case 0:
            if (ntrt1m_invalidViReal64Range (phaseScale, -2.0*Pi, 0.0))
                return VI_ERROR_PARAMETER7; 
            break;
        case 1:
            if (ntrt1m_invalidViReal64Range (phaseScale, -360.0, 0.0))
                return VI_ERROR_PARAMETER7; 
            break;
    }
    if (ntrt1m_invalidViBooleanRange (crosstalk))
        return VI_ERROR_PARAMETER8;

    switch (configure) {
        case 0:
            Fmt (wrtBuf, "%s<MEAS1:LEV:UNIT %s;MEAS1:DIST:UNIT %s;MEAS1:NOIS:UNIT %s;"
                        "MEAS:PHAS:UNIT %s;MEAS:PHAS:SCAL %f;MEAS1:CROS:UNIT %s;"
                        "MEAS2:LEV:UNIT %s;MEAS2:DIST:UNIT %s;MEAS2:NOIS:UNIT %s;"
                        "MEAS2:CROS:UNIT %s",
                fourUnits[levelUnit], twoUnits2[distortionUnit], twoUnits2[noiseUnit],
                angleUnits[phaseUnit], phaseScale, crossUnits[crosstalk],
                fourUnits[levelUnit], twoUnits2[distortionUnit], twoUnits2[noiseUnit],
                crossUnits[crosstalk]);
            break;
        case 1:
            Fmt (wrtBuf, "%s<MEAS1:LEV:UNIT %s;MEAS1:DIST:UNIT %s;MEAS1:NOIS:UNIT %s;"
                        "MEAS:PHAS:UNIT %s;MEAS:PHAS:SCAL %f;MEAS1:CROS:UNIT %s",
                fourUnits[levelUnit], twoUnits2[distortionUnit], twoUnits2[noiseUnit],
                angleUnits[phaseUnit], phaseScale, crossUnits[crosstalk]);
            break;
        case 2:
            Fmt (wrtBuf, "%s<MEAS2:LEV:UNIT %s;MEAS2:DIST:UNIT %s;MEAS2:NOIS:UNIT %s;"
                        "MEAS:PHAS:UNIT %S;MEAS:PHAS:SCAL %F;MEAS2:CROS:UNIT %S",
                fourUnits[levelUnit], twoUnits2[distortionUnit], twoUnits2[noiseUnit],
                angleUnits[phaseUnit], phaseScale, crossUnits[crosstalk]);
            break;
    }
    if ((ntrt1m_status = viWrite (instrSession, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)
        return ntrt1m_status;
    
    return ntrt1m_status;
}

/*===========================================================================*/
/* Function: Configure Multitone                                             */
/* Purpose:  This function selects the active signal to be used for          */
/*           transmission from 1 of 4 memory locations, and sets the         */
/*           durations of the pre-trigger signal and the multitone burst     */
/*===========================================================================*/
ViStatus _VI_FUNC ntrt1m_configMtone (ViSession instrSession,
                    ViInt16 activeSignal, ViReal64 pretriggerLengthms,
                    ViReal64 multitoneBurstLengthms)
{
    ViStatus ntrt1m_status = VI_SUCCESS;
    ViChar wrtBuf[BUFFER_SIZE];
    ViInt32 retCnt;
    
    if (ntrt1m_invalidViInt16Range (activeSignal,1,4))
        return VI_ERROR_PARAMETER2;
    if (ntrt1m_invalidViReal64Range (pretriggerLengthms,0.0,30000.0))
        return VI_ERROR_PARAMETER3;
    if (ntrt1m_invalidViReal64Range (multitoneBurstLengthms,0.0,30000.0))
        return VI_ERROR_PARAMETER4;

    Fmt (wrtBuf, "%s<OUTP:MTON:ACT %d[b2];OUTP:MTON:PRET %f;OUTP:MTON:MTON %f",
            activeSignal, pretriggerLengthms, multitoneBurstLengthms);
    if ((ntrt1m_status = viWrite (instrSession, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)
        return ntrt1m_status;

    return ntrt1m_status;
}

/*===========================================================================*/
/* Function: Define Multitone Signal                                         */
/* Purpose:  This function defines the user-specified multitone signal, and  */
/*           it stores the signal to the selected memory location.           */                   
/*===========================================================================*/
ViStatus _VI_FUNC ntrt1m_defineMtoneSignal (ViSession instrSession,
                    ViInt16 signalNumber, ViChar _VI_FAR signalName[],
                    ViInt16 numberofSamples, ViInt16 numberofBinsinCH1,
                    ViInt16 _VI_FAR CH1BinFreqHz[], ViReal64 _VI_FAR CH1BinPhase[],
                    ViInt16 numberofBinsinCH2, ViInt16 _VI_FAR CH2BinFreqHz[],
                    ViReal64 _VI_FAR CH2BinPhase[])
{
    ViStatus ntrt1m_status = VI_SUCCESS, ntrt1m_status1 = VI_SUCCESS;
    ViChar wrtBuf[LARGE_BUFFER_SIZE];
    ViInt32 retCnt, breakError;
    ViInt16 i;
    ViEvent eventHandle;
    ViUInt16    byteRd;
    ViEventType dummy1;
    ViInt16 SRQ = 0, count = 0;
    ViByte rdBuf[BUFFER_SIZE];

    /* Check the ranges of the parameters */
    if (ntrt1m_invalidViInt16Range (signalNumber,1,4))
        return VI_ERROR_PARAMETER2;
    if (ntrt1m_invalidBlocklength (numberofSamples))
        return VI_ERROR_PARAMETER4;
    if (ntrt1m_invalidViInt16Range (numberofBinsinCH1,1,31))
        return VI_ERROR_PARAMETER5;
    if (ntrt1m_invalidViInt16Range (numberofBinsinCH2,1,31))
        return VI_ERROR_PARAMETER8;

    /* Double check that the registers are cleared. 
       Note that this is actually done in the initialization function, too */
    if ((ntrt1m_status = viWrite (instrSession, "*SRE 0;*ESE 0", 13, &retCnt)) < 0)
        return ntrt1m_status;
    #if (!NOVISAEVENT)
    /* Read status byte to make sure the RQS bit isn't set
              before starting SRQ operation */
    if ((ntrt1m_status = viReadSTB (instrSession, &byteRd)) < 0)
        return ntrt1m_status;

    /* Enable events */
    if ((ntrt1m_status = viEnableEvent (instrSession, VI_EVENT_SERVICE_REQ, VI_QUEUE, VI_NULL)) < 0)
        return ntrt1m_status;   
    
    #endif
    /* Clear possible SRQ by reading event status register */
    if ((ntrt1m_status = viWrite (instrSession, "*ESR?", 5, &retCnt)) < 0)
        return ntrt1m_status;
    if ((ntrt1m_status = viRead (instrSession, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
        return ntrt1m_status;
    /* Enable operation complete event */
    if ((ntrt1m_status = viWrite (instrSession, "*SRE 32;*ESE 1", 14, &retCnt)) < 0)
        return ntrt1m_status;
    
    /* Read status byte to clear instrument RQS register  - for some reason it is set after writing
        *SRE 32 and *ESE 1  */
    if ((ntrt1m_status = viReadSTB (instrSession, &byteRd)) < 0)
             return ntrt1m_status;

    /* Discard any events in queue */
    if ((ntrt1m_status = viDiscardEvents (instrSession, VI_EVENT_SERVICE_REQ,
                                           VI_QUEUE)) < 0)
        return ntrt1m_status;

    /* Format the first 5 parameters */
    Fmt (wrtBuf, "%s<OUTP:MTON:PAR %d[b2],'%s',%d[b2],%d[b2],%d[b2]", signalNumber,
        signalName, numberofSamples, numberofBinsinCH1, numberofBinsinCH2);
    /* Format the bins in channel 1 and 2 */
    for (i = 0; i < numberofBinsinCH1; i++) {
        Fmt (wrtBuf, "%s<%s,%d[b2]", wrtBuf, CH1BinFreqHz[i]);    
    }
    for (i = 0; i < numberofBinsinCH2; i++) {
        Fmt (wrtBuf, "%s<%s,%d[b2]", wrtBuf, CH2BinFreqHz[i]);    
    }
    /* Format the phases in channel 1 and 2 */
    for (i = 0; i < numberofBinsinCH1; i++) {
        Fmt (wrtBuf, "%s<%s,%f", wrtBuf, CH1BinPhase[i]);    
    }
    for (i = 0; i < numberofBinsinCH2; i++) {
        Fmt (wrtBuf, "%s<%s,%f", wrtBuf, CH2BinPhase[i]);    
    }
    /* Add *OPC in the end of the string that defines this signal */
    Fmt (wrtBuf, "%s<%s;*OPC",wrtBuf);
    
    /* Send the signal information to the instrument */
    if ((ntrt1m_status = viWrite (instrSession, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)
        return ntrt1m_status;

    #if (NOVISAEVENT)
    while (!SRQ & (count < 60)) {
         /* Read status byte to clear instrument RQS register */
         if ((ntrt1m_status = viReadSTB (instrSession, &byteRd)) < 0)
             return ntrt1m_status;
         if (byteRd & 0x40)  // SRQ occurred regardless of viWaitOnEvent
            SRQ = 1;
        count++;
        Delay(1.0);
    }
    if ((!SRQ & (count > 60)) < 0)
        return VI_ERROR_TMO;
    #else
    /* Wait for SRQ for 60 seconds for operation to complete.*/
    /* Wait until the instrument finishes its calculation */

    if ((ntrt1m_status1 = viWaitOnEvent (instrSession, VI_EVENT_SERVICE_REQ, 60000, &dummy1,
                                 &eventHandle)) < 0) {
        ntrt1m_status = viReadSTB (instrSession, &byteRd);
        ntrt1m_status = viDisableEvent (instrSession, VI_EVENT_SERVICE_REQ, VI_QUEUE);
        ntrt1m_status = viWrite (instrSession, "*SRE 0;*ESE 0", 13, &retCnt);
        return ntrt1m_status1;
        }
    /* Close the event handle which was created by viWaitOnEvent */
    if((ntrt1m_status = viClose (eventHandle)) < 0)
        return ntrt1m_status;

    /* Read status byte to clear instrument RQS register */
    if ((ntrt1m_status = viReadSTB (instrSession, &byteRd)) < 0)
        return ntrt1m_status;
    /* Disable VISA events */
    if ((ntrt1m_status = viDisableEvent (instrSession, VI_EVENT_SERVICE_REQ, VI_QUEUE)) < 0)
        return ntrt1m_status;
   
   #endif 
   
    /* Clear registers on instrument */
    if ((ntrt1m_status = viWrite (instrSession, "*SRE 0;*ESE 0", 13, &retCnt)) < 0)
        return ntrt1m_status;
    
    return ntrt1m_status;
}

/*===========================================================================*/
/* Function: Start Active Multitone                                          */
/* Purpose:  This function causes the generator to start with the active     */
/*           multitone signal.                                               */                   
/*===========================================================================*/
ViStatus _VI_FUNC ntrt1m_startActiveMtone (ViSession instrSession)
{
    ViStatus ntrt1m_status = VI_SUCCESS;
    ViChar wrtBuf[BUFFER_SIZE];
    ViInt32 retCnt;
    
    if ((ntrt1m_status = viWrite (instrSession, "OUTP:MTON:STAR", 14, &retCnt)) < 0)
        return ntrt1m_status;

    return ntrt1m_status;
}

/*===========================================================================*/
/* Function: Continuous Active Multitone                                     */
/* Purpose:  This function causes the generator to start with the active     */
/*           multitone signal in an endless loop.                            */                   
/*===========================================================================*/
ViStatus _VI_FUNC ntrt1m_contActiveMtone (ViSession instrSession)
{
    ViStatus ntrt1m_status = VI_SUCCESS;
    ViChar wrtBuf[BUFFER_SIZE];
    ViInt32 retCnt;
    
    if ((ntrt1m_status = viWrite (instrSession, "OUTP:MTON:CON", 13, &retCnt)) < 0)
        return ntrt1m_status;

    return ntrt1m_status;
}

/*===========================================================================*/
/* Function: Read Measurement                                                */
/* Purpose:  This function reads either the measured amplitudes, the         */
/*           distortion, or the noise.  It returns the number of bins        */
/*           queried, the array that contains the bin numbers, and the array */
/*           that holds the data.                                            */                   
/*===========================================================================*/
ViStatus _VI_FUNC ntrt1m_readMeas (ViSession instrSession,
                    ViInt16 selectChannel, ViInt16 dataType, 
                    ViPInt16 numberofBinsQueried, ViInt16 _VI_FAR binNumber[],
                    ViReal64 _VI_FAR amplitude[], ViChar _VI_FAR unit[])
{
    ViStatus ntrt1m_status = VI_SUCCESS;
    ViChar wrtBuf[BUFFER_SIZE], rdBuf[LARGE_BUFFER_SIZE], nanStr[BUFFER_SIZE];
    ViInt32 retCnt, breakError;
    ViInt16 index, count, errorCode;
    ViString chPtr;

    if (ntrt1m_invalidViInt16Range (selectChannel,1,2))
        return VI_ERROR_PARAMETER2;
    if (ntrt1m_invalidViInt16Range (dataType,0,4))
        return VI_ERROR_PARAMETER3;

    Fmt (wrtBuf, "%s<MEAS%d[b2]:%s?", selectChannel, selMeas[dataType]);
    if ((ntrt1m_status = viWrite (instrSession, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)
        return ntrt1m_status;

    /* The instrument now has either data or ';' indicating an error*/
    if ((ntrt1m_status = viRead (instrSession, rdBuf, LARGE_BUFFER_SIZE, &retCnt)) < 0)
        return ntrt1m_status;
        
    if (rdBuf[0] == ';') {
         /* Check to see if read measurement function failed or not due to */
         /* overflow at input or no trigger detected */
         if ((ntrt1m_status = viWrite (instrSession, "SYST:ERR?", 9, &retCnt)) < 0)
             return ntrt1m_status;
        /* The instrument now has error 203 or 210*/
        if ((ntrt1m_status = viRead (instrSession, rdBuf, LARGE_BUFFER_SIZE, &retCnt)) < 0)
           return ntrt1m_status;
        if ((Scan(rdBuf, "%s>%d[b2]", &errorCode)) != 1)
            return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        if (errorCode != 0){
            switch (errorCode) {
                case 203:
                    ntrt1m_status = ntrt1m_ERROR_NO_TRIGGER_DETECTED;
                    return ntrt1m_status;
                case 210:
                    ntrt1m_status = ntrt1m_ERROR_OVERFLOW_AT_INPUT;
                    return ntrt1m_status;
                default:
                    ntrt1m_status = ntrt1m_SYSTEM_ERROR;
                    return ntrt1m_status;
            }
        }
    }    
    
    /* Read first bin number, the value, AND the unit. */    
    if ((Scan(rdBuf, "%s>%d[b2]/%f %s[xt44t10]", &binNumber[0], 
                                            &amplitude[0], unit)) != 3){
        if ((Scan(rdBuf, "%s>%d[b2]/%f %s[xt44t10]",&binNumber[0],
                                            nanStr, unit)) !=3)
            return VI_ERROR_INSTR_INTERPRETING_RESPONSE;   // Cannot convert
        chPtr = strstr(nanStr,"NAN");   // Check to see if String is NaN 
        
                                        
        if(chPtr)  {
            /* Need to set amplitude to NaN;  The easiest way to implement this is to
               set the value equal to sqrt(-1) -- but this will generate a library error  */
            breakError = SetBreakOnLibraryErrors (0);    
            amplitude[0] = sqrt(-1); // set value to NaN in amplitude array  
            SetBreakOnLibraryErrors(breakError);  // undoing ignore library errors
        }
        else 
            return VI_ERROR_INSTR_INTERPRETING_RESPONSE;  //string not a NaN val
    }
    count = 1;      /* First data set read successfully. */
    index = 0;      /* Initialize index */

    /* Read the rest of the data */
    while ((index = 1 + FindPattern (rdBuf, index, -1, ",", 0, 0)) != 0) {    
        if ((Scan(rdBuf, "%s[i*]>%d[b2]/%f", index,&binNumber[count],
                                            &amplitude[count])) != 2) {
            if ((Scan(rdBuf, "%s[i*]>%d[b2]/%s",index, &binNumber[count],
                                            nanStr)) !=2)
                return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
            chPtr = strstr(nanStr,"NAN");   // Check to see if String is NaN  
            if(chPtr) {
            /* Need to set amplitude to NaN;  The easiest way to implement this is to
               set the value equal to sqrt(-1) -- but this will generate a library error  */
            breakError = SetBreakOnLibraryErrors (0);    
            amplitude[count] = sqrt(-1); // set value to NaN in amplitude array  
            SetBreakOnLibraryErrors(breakError);  // undoing ignore library errors
            }
            else 
                return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        }
        count++;
    }
    *numberofBinsQueried = count;

     return ntrt1m_status;
}

/*===========================================================================*/
/* Function: Write To Instrument                                             */
/* Purpose:  This function writes a command string to the instrument.        */
/*===========================================================================*/
ViStatus _VI_FUNC ntrt1m_writeInstrData (ViSession instrSession, ViString writeBuffer)
{
    ViStatus ntrt1m_status = VI_SUCCESS;
    
    if ((ntrt1m_status = viPrintf (instrSession, "%s", writeBuffer)) < 0)
        return ntrt1m_status;

    return ntrt1m_status;
}

/*===========================================================================*/
/* Function: Read Instrument Buffer                                          */
/* Purpose:  This function reads the output buffer of the instrument.        */
/*===========================================================================*/
ViStatus _VI_FUNC ntrt1m_readInstrData (ViSession instrSession, ViInt32 numberBytesToRead,
                    ViChar _VI_FAR readBuffer[], ViPInt32 numBytesRead)
{
    ViStatus ntrt1m_status = VI_SUCCESS;
    *numBytesRead = 0L;
        
    if ((ntrt1m_status = viRead (instrSession, readBuffer, numberBytesToRead, numBytesRead)) < 0)
        return ntrt1m_status;

    return ntrt1m_status;
}

/*===========================================================================*/
/* Function: Reset                                                           */
/* Purpose:  This function resets the instrument.  If the reset function     */
/*           is not supported by the instrument, this function returns       */
/*           the warning VI_WARN_NSUP_RESET.                                 */
/*===========================================================================*/
ViStatus _VI_FUNC ntrt1m_reset (ViSession instrSession)
{
    ViUInt32 retCnt = 0;
    ViStatus ntrt1m_status = VI_SUCCESS;

    /*  Initialize the instrument to a known state.  */
    if ((ntrt1m_status = viWrite (instrSession, "*RST", 4, &retCnt)) < 0)
        return ntrt1m_status;

   //Delay (10);

    if ((ntrt1m_status = ntrt1m_defaultInstrSetup (instrSession)) < 0)  
        return ntrt1m_status;

    return ntrt1m_status;
}

/*===========================================================================*/
/* Function: Self-Test                                                       */
/* Purpose:  This function executes the instrument self-test and returns     */
/*           the result. If the self test function is not supported by the   */
/*           instrument, this function returns the warning                   */
/*           VI_WARN_NSUP_SELF_TEST.                                         */
/*===========================================================================*/
ViStatus _VI_FUNC ntrt1m_selfTest (ViSession instrSession, ViPInt16 testResult,
                    ViChar _VI_FAR testMessage[])
{
    ViUInt32 retCnt = 0;
    ViStatus ntrt1m_status = VI_SUCCESS;

    if ((ntrt1m_status = viWrite (instrSession, "*TST?", 5, &retCnt)) < 0)
        return ntrt1m_status;

    Delay (20.0);

    if ((ntrt1m_status = viScanf (instrSession, "%hd", testResult)) < 0)
        return ntrt1m_status;
    if (testResult == 0)
        Fmt (testMessage, "%s<Self-Test Passed");
    else
        Fmt (testMessage, "%s<See Manual for Self-Test Code Description");
    
    return ntrt1m_status;
}

/*===========================================================================*/
/* Function: Error Query                                                     */
/* Purpose:  This function queries the instrument error queue, and returns   */
/*           the result. If the error query function is not supported by the  */
/*===========================================================================*/
ViStatus _VI_FUNC ntrt1m_errorQuery (ViSession instrSession, ViPInt32 errorCode,
                    ViChar _VI_FAR errorMessage[])
{
    ViUInt32 retCnt = 0;
    ViStatus ntrt1m_status = VI_SUCCESS;

    if ((ntrt1m_status = viWrite (instrSession, "SYST:ERR?", 9, &retCnt)) < 0)
        return ntrt1m_status;
    if ((ntrt1m_status = viScanf (instrSession, "%ld", errorCode)) < 0)
        return ntrt1m_status;
    Fmt (errorMessage, "%s<See Manual for Error Code Description");

    return ntrt1m_status;
}
/*===========================================================================*/
/* Function: Error Message                                                   */
/* Purpose:  This function translates the error return value from the        */
/*           instrument driver into a user-readable string.                  */
/*===========================================================================*/
ViStatus _VI_FUNC ntrt1m_errorMessage (ViSession instrSession, ViStatus statusCode,
                    ViChar _VI_FAR errMessage[])
{
    ViStatus ntrt1m_status = VI_SUCCESS;
    ViInt16 i;
    static ntrt1m_tStringValPair statusDescArray[] = {
        {VI_WARN_NSUP_ID_QUERY,     "WARNING: ID Query not supported"},
        {VI_WARN_NSUP_RESET,        "WARNING: Reset not supported"},
        {VI_WARN_NSUP_SELF_TEST,    "WARNING: Self-test not supported"},
        {VI_WARN_NSUP_ERROR_QUERY,  "WARNING: Error Query not supported"},     
        {VI_WARN_NSUP_REV_QUERY,    "WARNING: Revision Query not supported"},
        {VI_ERROR_PARAMETER1,   "ERROR: Parameter 1 out of range"},
        {VI_ERROR_PARAMETER2,   "ERROR: Parameter 2 out of range"},
        {VI_ERROR_PARAMETER3,   "ERROR: Parameter 3 out of range"},
        {VI_ERROR_PARAMETER4,   "ERROR: Parameter 4 out of range"},
        {VI_ERROR_PARAMETER5,   "ERROR: Parameter 5 out of range"},
        {VI_ERROR_PARAMETER6,   "ERROR: Parameter 6 out of range"},
        {VI_ERROR_PARAMETER7,   "ERROR: Parameter 7 out of range"},
        {VI_ERROR_PARAMETER8,   "ERROR: Parameter 8 out of range"},
        {VI_ERROR_FAIL_ID_QUERY,"ERROR: Identification query failed"},
        {VI_ERROR_INV_RESPONSE, "ERROR: Interpreting instrument response"},
        {VI_ERROR_INSTR_FILE_OPEN,    "ERROR: Opening the specified file"},
        {VI_ERROR_INSTR_FILE_WRITE,   "ERROR: Writing to the specified file"},
        {VI_ERROR_INSTR_INTERPRETING_RESPONSE, "ERROR: Interpreting the instrument's response"},
        {VI_ERROR_INSTR_PARAMETER9 ,  "ERROR: Parameter 9 out of range"},
        {VI_ERROR_INSTR_PARAMETER10,  "ERROR: Parameter 10 out of range"},
        {VI_ERROR_INSTR_PARAMETER11,  "ERROR: Parameter 11 out of range"},
        {VI_ERROR_INSTR_PARAMETER12,  "ERROR: Parameter 12 out of range"},
        {VI_ERROR_INSTR_PARAMETER13,  "ERROR: Parameter 13 out of range"},
        {VI_ERROR_INSTR_PARAMETER14,  "ERROR: Parameter 14 out of range"},
        {VI_ERROR_INSTR_PARAMETER15,  "ERROR: Parameter 15 out of range"},
                
        {ntrt1m_ERROR_INVALID_CONFIGURATION, "ERROR: Instrument configuration error"},
        {ntrt1m_WARNING_NAN_RETURNED,       "WARNING: At least one value was returned as NaN"},
        {ntrt1m_ERROR_NO_TRIGGER_DETECTED,  "ERROR: No trigger was detected"},
        {ntrt1m_ERROR_OVERFLOW_AT_INPUT,    "ERROR: Overflow at input"},
        {ntrt1m_SYSTEM_ERROR,               "ERROR: System error"},
        {VI_NULL, VI_NULL}
    };

    ntrt1m_status = viStatusDesc (instrSession, statusCode, errMessage);
    if (ntrt1m_status == VI_WARN_UNKNOWN_STATUS) {
        for (i = 0; statusDescArray[i].stringName; i++) {
            if (statusDescArray[i].stringVal == statusCode) {
                Fmt (errMessage, "%s<%s", statusDescArray[i].stringName);
                return (VI_SUCCESS);
            }
        }
        Fmt (errMessage, "%s<Unknown Error 0x%x[uw8p0]", statusCode);
        return (VI_WARN_UNKNOWN_STATUS);
    }
    
    ntrt1m_status = VI_SUCCESS;
    return ntrt1m_status;
}

/*===========================================================================*/
/* Function: Revision Query                                                  */
/* Purpose:  This function returns the driver and instrument revisions.      */
/*           If the revision query function is not supported by the          */ 
/*           instrument, this function returns the warning                   */
/*           VI_WARN_NSUP_REV_QUERY.                                         */
/*===========================================================================*/
ViStatus _VI_FUNC ntrt1m_revisionQuery (ViSession instrSession,
                    ViChar _VI_FAR driverRev[], ViChar _VI_FAR instrRev[])
{
    ViUInt32 retCnt = 0;
    ViStatus ntrt1m_status = VI_SUCCESS;

    if ((ntrt1m_status = viWrite (instrSession, "*IDN?", 5, &retCnt)) < 0)
        return ntrt1m_status;
    if ((ntrt1m_status = viScanf (instrSession, "%*[^,],%*[^,],%*[^,],%[^\n]", instrRev)) < 0)
        return ntrt1m_status;

    Fmt (driverRev, "%s<%s", ntrt1m_REVISION);
    
    return ntrt1m_status;
}

/*===========================================================================*/
/* Function: Close                                                           */
/* Purpose:  This function closes the instrument.                            */
/*===========================================================================*/
ViStatus _VI_FUNC ntrt1m_close (ViSession instrSession)
{
    ntrt1m_instrRange instrPtr;
    ViSession rmSession;
    ViStatus ntrt1m_status = VI_SUCCESS;

    if ((ntrt1m_status = viGetAttribute (instrSession, VI_ATTR_RM_SESSION, &rmSession)) < 0)
        return ntrt1m_status;
    if ((ntrt1m_status = viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr)) < 0)
        return ntrt1m_status;
            
    if (instrPtr != NULL) 
        free (instrPtr);
    
    ntrt1m_status = viClose (instrSession);
    viClose (rmSession);

    return ntrt1m_status;
}
/*****************************************************************************/
/*= UTILITY ROUTINES (Non-Exportable Functions) =============================*/
/*****************************************************************************/

/*===========================================================================*/
/* Function: Boolean Value Out Of Range - ViBoolean                          */
/* Purpose:  This function checks a Boolean to see if it is equal to VI_TRUE */
/*           or VI_FALSE. If the value is out of range, the return value is  */
/*           VI_TRUE, otherwise the return value is VI_FALSE.                */
/*===========================================================================*/
ViBoolean ntrt1m_invalidViBooleanRange (ViBoolean val)
{
    return ((val != VI_FALSE && val != VI_TRUE) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Short Signed Integer Value Out Of Range - ViInt16               */
/* Purpose:  This function checks a short signed integer value to see if it  */  
/*           lies between a minimum and maximum value.  If the value is out  */
/*           of range, the return value is VI_TRUE, otherwise the return     */
/*           value is VI_FALSE.                                              */
/*===========================================================================*/
ViBoolean ntrt1m_invalidViInt16Range (ViInt16 val, ViInt16 min, ViInt16 max)
{
    return ((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Long Signed Integer Value Out Of Range - ViInt32                */
/* Purpose:  This function checks a long signed integer value to see if it   */  
/*           lies between a minimum and maximum value.  If the value is out  */
/*           of range, the return value is VI_TRUE, otherwise the return     */
/*           value is VI_FALSE.                                              */
/*===========================================================================*/
ViBoolean ntrt1m_invalidViInt32Range (ViInt32 val, ViInt32 min, ViInt32 max)
{
    return ((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Unsigned Char Value Out Of Range - ViUInt8                      */
/* Purpose:  This function checks an unsigned char value to see if it        */  
/*           lies between a minimum and maximum value.  If the value is out  */
/*           of range, the return value is VI_TRUE, otherwise the return     */
/*           value is VI_FALSE.                                              */
/*===========================================================================*/
ViBoolean ntrt1m_invalidViUInt8Range (ViUInt8 val, ViUInt8 min, ViUInt8 max)
{
    return ((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Short Unsigned Integer Value Out Of Range - ViUInt16            */
/* Purpose:  This function checks a short unsigned integer value to see if it*/  
/*           lies between a minimum and maximum value.  If the value is out  */
/*           of range, the return value is VI_TRUE, otherwise the return     */
/*           value is VI_FALSE.                                              */
/*===========================================================================*/
ViBoolean ntrt1m_invalidViUInt16Range (ViUInt16 val, ViUInt16 min, ViUInt16 max)
{
    return ((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Long Unsigned Integer Value Out Of Range - ViUInt32             */
/* Purpose:  This function checks a long unsigned integer value to see if it */  
/*           lies between a minimum and maximum value.  If the value is out  */
/*           of range, the return value is VI_TRUE, otherwise the return     */
/*           value is VI_FALSE.                                              */
/*===========================================================================*/
ViBoolean ntrt1m_invalidViUInt32Range (ViUInt32 val, ViUInt32 min, ViUInt32 max)
{
    return ((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Real (Float) Value Out Of Range - ViReal32                      */
/* Purpose:  This function checks a real (float) value to see if it lies     */  
/*           between a minimum and maximum value.  If the value is out of    */
/*           range, the return value is VI_TRUE, otherwise the return value  */
/*           is VI_FALSE.                                                    */
/*===========================================================================*/
ViBoolean ntrt1m_invalidViReal32Range (ViReal32 val, ViReal32 min, ViReal32 max)
{
    return ((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Real (Double) Value Out Of Range - ViReal64                     */
/* Purpose:  This function checks a real (double) value to see if it lies    */  
/*           between a minimum and maximum value.  If the value is out of    */
/*           range, the return value is VI_TRUE, otherwise the return value  */
/*           is VI_FALSE.                                                    */
/*===========================================================================*/
ViBoolean ntrt1m_invalidViReal64Range (ViReal64 val, ViReal64 min, ViReal64 max)
{
    return ((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Blocklength (Number of Samples) Out Of Range - ViInt16          */
/* Purpose:  This function checks a short signed integer value to see if it  */  
/*           is one of 5 allowed blocklength (512, 1024, 2048, 4096, 8192).  */
/*           If the value is none of the above, the return value is VI_TRUE, */
/*           otherwise the return value is VI_FALSE.                         */
/*===========================================================================*/
ViBoolean ntrt1m_invalidBlocklength (ViInt16 val)
{
    return ((val != 512 && val != 1024 && val != 2048 && val != 4096 &&
             val != 8192) ? VI_TRUE : VI_FALSE);
}

/*================================================================================*/
/* Function: Wait On VISA Event                                                   */
/* Purpose:  This function waits for the occurrence of an enabled VISA Event. The */
/*           event type must be enabled before entering this function. Any        */
/*           programmatic commands, actions, or conditions, necessary to generate */
/*           the specified event type, must be satisified before entering this    */
/*           function. The function will wait for the specified timeout and then  */
/*           return. If the specified event is received then the function will    */
/*           return VI_SUCCESS (0), otherwise the status code for the generated   */
/*           error will be returned. In either case the function will disable the */
/*           event type and deallocate the event handle passed from viWaitOnEvent */
/*           before returning. WARNING: If VI_TMO_INFINITE is passed in as the    */
/*           timeout this function WILL NOT return until the specified event is   */
/*           received, if the event is not received the function will not return  */
/*           and it will be necessary to terminate CVI in order to regain control.*/
/*================================================================================*/
ViStatus ntrt1m_waitOnVisaEvent (ViSession instrSession, ViEventType eventTypeIn, 
            ViUInt32 timeoutIn, ViPUInt16 STB)
{            
    ViStatus ntrt1m_status = VI_SUCCESS, tempStatus = VI_SUCCESS;
    ViEventType eventTypeOut = 0;
    ViEvent eventHandle = 0;
    
    /* For debug purposes we want to be able to see the status returned by        */
    /* viDisableEvent and viClose if one of the internal functions fails but do   */
    /* not want to return that value as that is not where the initial error       */
    /* occurs in the function, so we assign it to tempStatus.                     */
    
    if ((ntrt1m_status = viWaitOnEvent (instrSession, eventTypeIn, timeoutIn, 
                                        &eventTypeOut, &eventHandle)) < 0) {
        tempStatus = viDisableEvent (instrSession, eventTypeIn, VI_QUEUE);
        return ntrt1m_status;
    } 
    
    if (eventTypeIn == VI_EVENT_SERVICE_REQ)
        if ((ntrt1m_status = viReadSTB (instrSession, STB)) < 0) {
            tempStatus = viClose (eventHandle);
            tempStatus = viDisableEvent (instrSession, eventTypeIn, VI_QUEUE);
            return ntrt1m_status;
        }   
        
    if ((ntrt1m_status = viClose (eventHandle)) < 0) {             
        tempStatus = viDisableEvent (instrSession, eventTypeIn, VI_QUEUE);
        return ntrt1m_status;                 
    }
    
    if ((ntrt1m_status = viDisableEvent (instrSession, eventTypeIn, VI_QUEUE)) < 0)
        return ntrt1m_status;                                                   

    return ntrt1m_status;                                    
}    

/*===========================================================================*/
/* Function: Initialize Clean Up                                             */
/* Purpose:  This function is used only by the ntrt1m_init function.  When   */
/*           an error is detected this function is called to close the       */
/*           open resource manager and instrument object sessions and to     */
/*           set the instrSession that is returned from ntrt1m_init to       */
/*           VI_NULL.                                                        */
/*===========================================================================*/
ViStatus ntrt1m_initCleanUp (ViSession openRMSession,
                    ViPSession openInstrSession, ViStatus currentStatus)
{
    ntrt1m_instrRange instrPtr;
    
    if (viGetAttribute (*openInstrSession, VI_ATTR_USER_DATA, &instrPtr) >= 0)
        if (instrPtr != NULL) 
            free (instrPtr);

    viClose (*openInstrSession);
    viClose (openRMSession);
    *openInstrSession = VI_NULL;
    
    return currentStatus;
}

/*===========================================================================*/
/* Function: Read To File From Instrument                                    */
/* Purpose:  This function is used to read data from the instrument and      */
/*           write it to a user specified file.                              */
/*===========================================================================*/
ViStatus ntrt1m_readToFile (ViSession instrSession, ViString filename,
                    ViUInt32 readBytes, ViPUInt32 retCount)
{
    ViStatus  ntrt1m_status = VI_SUCCESS;
    ViByte    buffer[BUFFER_SIZE];
    ViUInt32  bytesReadInstr = 0, bytesWrittenFile = 0;
    FILE     *targetFile;

    *retCount = 0L;
    if ((targetFile = fopen (filename, "wb")) == VI_NULL)
        return VI_ERROR_INSTR_FILE_OPEN; /* not defined by VTL */

    for (;;) {
        if (readBytes > BUFFER_SIZE)
            ntrt1m_status = viRead (instrSession, buffer, BUFFER_SIZE, &bytesReadInstr);
        else
            ntrt1m_status = viRead (instrSession, buffer, readBytes, &bytesReadInstr);

        bytesWrittenFile = fwrite (buffer, sizeof (ViByte), (size_t)bytesReadInstr, targetFile);
        *retCount += bytesWrittenFile;
        if (bytesWrittenFile < bytesReadInstr)
            ntrt1m_status = VI_ERROR_INSTR_FILE_WRITE; /* not defined by VTL */

        if ((readBytes <= BUFFER_SIZE) || (ntrt1m_status <= 0) || (ntrt1m_status == VI_SUCCESS_TERM_CHAR))
            break;

        readBytes -= BUFFER_SIZE;
    }

    fclose (targetFile);
    return ntrt1m_status;
}

/*===========================================================================*/
/* Function: Write From File To Instrument                                   */
/* Purpose:  This function is used to read data from a user specified file   */
/*           and write it to the instrument.                                 */
/*===========================================================================*/
ViStatus ntrt1m_writeFromFile (ViSession instrSession, ViString filename,
                    ViUInt32 writeBytes, ViPUInt32 retCount)
{
    ViStatus  ntrt1m_status = VI_SUCCESS;
    ViByte    buffer[BUFFER_SIZE];
    ViUInt32  bytesRead = 0, bytesWritten = 0;
    FILE     *sourceFile;
    ViBoolean sendEnd = VI_FALSE;

    *retCount = 0L;
    if ((sourceFile = fopen (filename, "rb")) == VI_NULL)
        return VI_ERROR_INSTR_FILE_OPEN; /* not defined by VTL */

    while (!feof (sourceFile)) {
        bytesRead = (ViUInt32)fread (buffer, sizeof (ViByte), BUFFER_SIZE, sourceFile);
        if ((writeBytes > BUFFER_SIZE) && (bytesRead == BUFFER_SIZE)) {
            viGetAttribute (instrSession, VI_ATTR_SEND_END_EN, &sendEnd);
            viSetAttribute (instrSession, VI_ATTR_SEND_END_EN, VI_FALSE);
            ntrt1m_status = viWrite (instrSession, buffer, BUFFER_SIZE, &bytesWritten);
            viSetAttribute (instrSession, VI_ATTR_SEND_END_EN, sendEnd);
            writeBytes -= BUFFER_SIZE;
            *retCount += bytesWritten;
            if (ntrt1m_status < 0)
                break;
        }
        else {
            ntrt1m_status = viWrite (instrSession, buffer, ((bytesRead < writeBytes) ? bytesRead : writeBytes), &bytesWritten);
            *retCount += bytesWritten;
            break;
        }
    }

    fclose (sourceFile);
    return ntrt1m_status;
}

/*****************************************************************************/
/*----------- INSERT INSTRUMENT-DEPENDENT UTILITY ROUTINES HERE -------------*/
/*****************************************************************************/

/*===========================================================================*/
/* Function: Default Instrument Setup                                        */
/* Purpose:  This function sends a default setup to the instrument.  This    */
/*           function is called by the ntrt1m_reset operation and by the     */
/*           ntrt1m_init function if the reset option has not been           */
/*           selected.  This function is useful for configuring any          */
/*           instrument settings that are required by the rest of the        */
/*           instrument driver functions such as turning headers ON or OFF   */
/*           or using the long or short form for commands, queries, and data.*/                                    
/*===========================================================================*/
ViStatus ntrt1m_defaultInstrSetup (ViSession instrSession)
{
    ViStatus ntrt1m_status = VI_SUCCESS;
    ViUInt32 retCnt = 0;
    ntrt1m_instrRange instrPtr;
    
    /* Determine if the structure has been initialized for the current VISA  */
    /* Session and malloc if it has not.                                     */
    if (ntrt1m_status = viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr))
        return ntrt1m_status;
    
    if (instrPtr == NULL) 
        instrPtr = malloc (sizeof (struct ntrt1m_statusDataRanges));

    /*=CHANGE:=============================================================*/
    /* Change to reflect the global status variables that your driver needs*/
    /* to keep track of.  For example trigger mode of each session         */
    /*=====================================================================*/
    instrPtr -> triggerMode = 0;
    instrPtr -> val2 = 0;
    instrPtr -> val3 = 0;
    Fmt (instrPtr -> instrDriverRevision, "%s<%s", ntrt1m_REVISION);
    
    if ((ntrt1m_status = viSetAttribute (instrSession, VI_ATTR_USER_DATA, 
                            (ViUInt32)instrPtr)) < 0)
        return ntrt1m_status;
    if ((ntrt1m_status = viWrite (instrSession, "*SRE 0;*ESE 0", 13, &retCnt)) < 0)
        return ntrt1m_status;
 
    return ntrt1m_status;
}

/*****************************************************************************/
/*=== END INSTRUMENT DRIVER SOURCE CODE =====================================*/
/*****************************************************************************/
